/* *****************************************************************
Demo program to check installation.
****************************************************************** */

/*
# For local debugging and profiling:
# biddy> make profile "BINDIR = ./bin"
# est-2ed/source> make locallib (UNCOMMENT DEBUG+PROFILE IN MAKEFILE.LINUX)
# est-2ed/projects/demo> make local
# est-2ed/projects/demo> ./demo
# est-2ed/projects/demo> gprof demo gmon.out > analysis.txt
*/

#include <bdd.h>
#include <pa.h>
#include <versis.h>
#include <mc.h>
#include <strucval.h>
#include <ccs.h>

static void test1();
static void test2();

void hello() {
  printf("=======================================================================\n");
  printf("   Demo program from EST project.\n");
  printf("=======================================================================\n");
}

int main(int argc, char **argv)
{
  setbuf(stdout, NULL);

  hello();

  Bdd_InitPkg();
  Pa_InitPkg();
  Versis_InitPkg();
  Mc_InitPkg();
  Strucval_InitPkg();
  Ccs_InitPkg();

  Bdd_AboutPkg();
  Pa_AboutPkg();
  Versis_AboutPkg();
  Mc_AboutPkg();
  Strucval_AboutPkg();
  Ccs_AboutPkg();

  /* THIS PART IS FOR DEBUGGING */
  test1();

  /* THIS PART IS FOR PROFILING */
  test2();

  Ccs_ExitPkg();
  Strucval_ExitPkg();
  Mc_ExitPkg();
  Versis_ExitPkg();
  Pa_ExitPkg();
  Bdd_ExitPkg();

  printf("Demo finished.\n");

  return(0);
}

/* ******************************************************* */
/* LOCAL FUNCTIONS - it is a good idea to make them static */
/* ******************************************************* */

static void
test1()
{
  Bdd_Edge f,g;

  f = Bdd_Eval1("(AND X Y)");
  Bdd_SaveFormula(NULL,"F",f);
  Bdd_WriteTable("F");

  g = Bdd_Eval1("(OR F (AND V W))");
  Bdd_SaveFormula(NULL,"G",g);
  Bdd_WriteBDD("G");
}

static void
test2()
{
  /* compare this function with ../../data/filozofi/philo-test.tcl */

  Pa_ParseSortProcess("../../data/filozofi/philo5.dat");
  Pa_EncodeMultiProcesses("FLZF1 VLC1 FLZF2 VLC2 FLZF3 VLC3 FLZF4 VLC4 FLZF5 VLC5",NULL);
  printf("REPORT BEFORE Versis_Compose\n"); Bdd_SystemStat(NULL);
  Versis_Compose("S",1,
                 "FLZF1 VLC1 FLZF2 VLC2 FLZF3 VLC3 FLZF4 VLC4 FLZF5 VLC5",
                 "think1 think2 think3 think4 think5 eat1 eat2 eat3 eat4 eat5",
                 FALSE);
  printf("REPORT AFTER Versis_Compose\n"); Bdd_SystemStat(NULL);
  Ccs_ParseCCS("../../data/filozofi/philo5dat-v1.ccs",TRUE);
  Ccs_ParseCCS("../../data/filozofi/philo5dat-v2.ccs",TRUE);
  printf("REPORT AFTER BOTH Ccs_ParseCCS\n"); Bdd_SystemStat(NULL);
  printf("STARTED FIRST EQUIVALENCE CHECKING, PLEASE WAIT...\n");  
  if (Versis_Equivalence(0,0,"S1",1,"S")) {
    printf("S1 is strong equivalent to S\n");  
  } else {    
    printf("S1 is not strong equivalent to S\n");  
  }
  printf("REPORT AFTER FIRST Versis_Equivalence\n"); Bdd_SystemStat(NULL);
  printf("STARTED SECOND EQUIVALENCE CHECKING, PLEASE WAIT...\n");  
  if (Versis_Equivalence(0,0,"S2",1,"S")) {
    printf("S2 is strong equivalent to S\n");  
  } else {    
    printf("S2 is not strong equivalent to S\n");  
  }
  printf("REPORT AFTER SECOND Versis_Equivalence\n"); Bdd_SystemStat(NULL);
  printf("STARTED THIRD EQUIVALENCE CHECKING, PLEASE WAIT...\n");  
  if (Versis_Equivalence(0,0,"S1",0,"S2")) {
    printf("S1 is strong equivalent to S2\n");  
  } else {    
    printf("S1 is not strong equivalent to S2\n");  
  }
  printf("REPORT AFTER THIRD Versis_Equivalence\n"); Bdd_SystemStat(NULL);
}