/**CHeaderFile*************************************************************
  PackageName [bdd]
  Synopsis    [Package 'bdd' enable symbolic computations by representing
               Boolean functions with ROBDDs.]

  FileName    [bddInt.h]
  Revision    [$Revision: 124 $]
  Date        [$Date: 2016-06-19 21:16:51 +0200 (ned, 19 jun 2016) $]
  Authors     [Robert Meolic (robert.meolic@um.si)]
  Description [File bddInt.h contains declaration of internal
               data structures.]
  SeeAlso     [bdd.h]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 2003, 2016
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

#ifndef _BDDINT
#define _BDDINT

#include "bdd.h"

/* if using sockets then printf("\n") will produce two new lines */
/* you will avoid this efect by using printf(NEWLINE) which adds extra space */
/* this extra space is not needed if counting bytes mechanism is used */
#ifdef USESOCKETS
#  include <gui.h>
#  ifdef USE_GUI_STUBS
#    include <guiDecls.h>
#  endif
#  ifdef ESTWIN32
#    define printf(...) {char sendbuf[1024]; sprintf(sendbuf,__VA_ARGS__);if(gui_outputLength<0){send(gui_outputStream,sendbuf,(int)strlen(sendbuf),0);}else{gui_outputLength+=send(gui_outputStream,sendbuf,(int)strlen(sendbuf),0);}send(gui_outputStream,"\n",1,0);Gui_Update();}
#  else
#    define printf(...) {if(gui_outputLength<0){fprintf(gui_outputStream,__VA_ARGS__);}else{gui_outputLength+=fprintf(gui_outputStream,__VA_ARGS__);}fprintf(gui_outputStream,"\n");Gui_Update();}
#    define debugprintf(...) {fprintf(stderr,"BDD[");fprintf(stderr,__VA_ARGS__);fprintf(stderr,"]");if(gui_outputLength<0){fprintf(gui_outputStream,__VA_ARGS__);}else{gui_outputLength+=fprintf(gui_outputStream,__VA_ARGS__);}fprintf(gui_outputStream,"\n");Gui_Update();}
#  endif
#  define NEWLINE " \n"
#else
#  define NEWLINE "\n"
#endif

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Structure declarations                                                */
/*-----------------------------------------------------------------------*/

/* RelOp Cache */
typedef struct {
  unsigned int key;            /* 0 = not valid node! */
  Bdd_Edge f;
  Bdd_Edge g;
  Bdd_Edge r;
} BddRelOpCache;

typedef struct {
  BddRelOpCache *table;
  unsigned int size;
  unsigned long long int num; /* number of non-recursive cals */
  unsigned long long int search; /* number of cache search */
  unsigned long long int find; /* number of cache find */
  unsigned long long int overwrite; /* number of cache overwrite */
} BddRelOpCacheTable;

/* RelOpSimple Cache */
typedef struct {
  unsigned int key;            /* 0 = not valid node! */
  Bdd_Edge f;
  Bdd_Edge r;
} BddRelOpSimpleCache;

typedef struct {
  BddRelOpSimpleCache *table;
  unsigned int size;
  unsigned long long int num; /* number of non-recursive cals */
  unsigned long long int search;
  unsigned long long int find;
  unsigned long long int overwrite;
} BddRelOpSimpleCacheTable;

/* RelOpComplex Cache */
typedef struct {
  unsigned int key;            /* 0 = not valid node! */
  Bdd_Edge f;
  Bdd_Edge r;
  Est_String name;
} BddRelOpComplexCache;

typedef struct {
  BddRelOpComplexCache *table;
  unsigned int size;
  unsigned long long int num;  /* number of non-recursive cals */
  unsigned long long int search;
  unsigned long long int find;
  unsigned long long int overwrite;
} BddRelOpComplexCacheTable;

/* StabRS Cache */
typedef struct {
  Bdd_Edge f;
  Bdd_Edge stablist;
  Bdd_Edge r;                  /* bdd_termNull = not valid record! */
} BddStabRSCache;

typedef struct {
  BddStabRSCache *table;
  unsigned int size;
  unsigned long long int search; /* number of cache search */
  unsigned long long int find; /* number of cache find */
  unsigned long long int overwrite; /* number of cache overwrite */
} BddStabRSCacheTable;

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

extern BddRelOpCacheTable bddRelOpCache;

extern BddRelOpSimpleCacheTable bddRelOpSimpleCache;

extern BddRelOpComplexCacheTable bddRelOpComplexCache;

extern BddStabRSCacheTable bddStabRSCache;

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Function prototypes                                                   */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* ALL INTERNAL FUNCTIONS ARE DEFINED IN bddMain.c                       */
/*-----------------------------------------------------------------------*/

extern Est_Boolean BddisOK(Bdd_Edge f);

extern void BddSelectNode(Bdd_Edge f);

extern void BddDeselectNode(Bdd_Edge f);

extern Est_Boolean BddisSelected(Bdd_Edge f);

extern void BddNodeSelect(Bdd_Edge f);

extern void BddNodeRepair(Bdd_Edge f);

extern Bdd_Edge BddFoaNode(Bdd_Variable v, Bdd_Edge pf, Bdd_Edge pt);

extern void BddExchange(Bdd_Edge *f, Bdd_Edge *g);

extern unsigned int BddNodeTableSize();

extern unsigned int BddNodeTableMax();

extern unsigned int BddNodeTableNum();

extern unsigned int BddNodeTableGarbage();

extern unsigned int BddNodeTableGenerated();

extern unsigned int BddNodeTableBlockNumber();

extern unsigned int BddNodeTableSwapNumber();

extern unsigned int BddNodeTableSiftingNumber();

extern unsigned int BddListUsed();

extern unsigned int BddListMaxLength();

extern float BddListAvgLength();

extern unsigned long long int BddIteCacheSearch();

extern unsigned long long int BddIteCacheFind();

extern unsigned long long int BddIteCacheOverwrite();

extern unsigned long long int BddRelopCacheNum();

extern unsigned long long int BddRelopCacheSearch();

extern unsigned long long int BddRelopCacheFind();

extern unsigned long long int BddRelopCacheOverwrite();

extern unsigned long long int BddRelopSimpleCacheNum();

extern unsigned long long int BddRelopSimpleCacheSearch();

extern unsigned long long int BddRelopSimpleCacheFind();

extern unsigned long long int BddRelopSimpleCacheOverwrite();

extern unsigned long long int BddRelopComplexCacheNum();

extern unsigned long long int BddRelopComplexCacheSearch();

extern unsigned long long int BddRelopComplexCacheFind();

extern unsigned long long int BddRelopComplexCacheOverwrite();

extern unsigned long long int BddStabRSCacheSearch();

extern unsigned long long int BddStabRSCacheFind();

extern unsigned long long int BddStabRSCacheOverwrite();

extern unsigned int BddNodeNumber(Bdd_Edge f);

extern unsigned int BddNodeNumberPlain(Bdd_Edge f);

extern unsigned int BddNodeMaxLevel(Bdd_Edge f);

extern float BddNodeAvgLevel(Bdd_Edge f);

extern unsigned int BddVariableNumber(Bdd_Edge f);

extern double BddMintermCount(Bdd_Edge f, int nvars);

/**AutomaticEnd***********************************************************/

#endif  /* _BDDINT */
