/**CFile*******************************************************************
  PackageName [bdd]
  Synopsis    [Package 'bdd' enable symbolic computations by representing
               Boolean functions with ROBDDs.]

  FileName    [bddTcl.c]
  Revision    [$Revision: 124 $]
  Date        [$Date: 2016-06-19 21:16:51 +0200 (ned, 19 jun 2016) $]
  Authors     [Robert Meolic (robert.meolic@um.si),
               Ales Casar (ales@homemade.net)]
  Description [File bddTcl.c contains definitions of Tcl commands,
               which can be used for manipulating with BDDs from
               Tcl interpreter (e.g. tclsh or wish).]
  SeeAlso     [bdd.h, bddInt.h]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 2003, 2016
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

#include "bddInt.h"
#include "bddDecls.h"

#include <tcl.h>

extern BddStubs bddStubs;
extern CONST char *Gui_InitStubs (Tcl_Interp *interp, char *version, int exact);

/* on tcl 8.3 use #define USECONST */
/* on tcl 8.4 use #define USECONST const*/
/* this is defined in Makefile */

/*-----------------------------------------------------------------------*/
/* Constant declarations                                                 */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Static function prototypes                                            */
/*-----------------------------------------------------------------------*/

static int BddInitPkgCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddExitPkgCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddAboutPkgCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddTerminalsCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddEval1Cmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddEval2Cmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddNOTCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddITECmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddECmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddACmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddRestrictCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddComposeCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddOutBDDCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddOutTableCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddOutFunctionCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddDrawBDDCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddCmpCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddFindCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddSupportCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddTableSizeCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddTableNumberCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddTableFreshCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddTableBadCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddTableMaxCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddTableGeneratedCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddBlockNumberCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddGarbageNumberCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddSwapNumberCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddSiftingNumberCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddListUsedCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddListMaxLengthCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddListAvgLengthCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddIteCacheSearchCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddIteCacheFindCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddIteCacheOverwriteCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddRelopCacheNumCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddRelopCacheSearchCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddRelopCacheFindCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddRelopCacheOverwriteCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddRelopSimpleCacheNumCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddRelopSimpleCacheSearchCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddRelopSimpleCacheFindCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddRelopSimpleCacheOverwriteCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddRelopComplexCacheNumCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddRelopComplexCacheSearchCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddRelopComplexCacheFindCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddRelopComplexCacheOverwriteCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddStabRSCacheSearchCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddStabRSCacheFindCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddStabRSCacheOverwriteCmd(ClientData clientData, Tcl_Interp *interp,
                            int argc, USECONST char **argv);

static int BddNodeNumberCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddNodeNumberPlainCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddNodeMaxLevelCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddNodeAvgLevelCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int BddVariableNumberCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);
                      
static int BddMintermCountCmd(ClientData clientData, Tcl_Interp *interp,
                          int argc, USECONST char **argv);

static int wordNumber(Est_String s);

/**AutomaticEnd***********************************************************/

/*-----------------------------------------------------------------------*/
/* Definition of exported functions                                      */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function BDD_Init.]
  Description [This function is used by Tcl/Tk only.]
  SideEffects []
  SeeAlso     []
  ************************************************************************/

#ifdef __cplusplus
extern "C" {
#endif

int
Bdd_Init(Tcl_Interp *interp)
{

#ifdef USE_TCL_STUBS
  if (Tcl_InitStubs(interp, "8.1", 0) == NULL) {
    return TCL_ERROR;
  }
#endif

#ifdef USE_GUI_STUBS
  if (Gui_InitStubs(interp, "1.0", 0) == NULL) {
    return TCL_ERROR;
  }
#endif

  Tcl_CreateCommand(interp, "bdd_initPkg", BddInitPkgCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_exitPkg", BddExitPkgCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_aboutPkg", BddAboutPkgCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_terminals", BddTerminalsCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_eval1", BddEval1Cmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_eval2", BddEval2Cmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_NOT", BddNOTCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_ITE", BddITECmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_E", BddECmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_A", BddACmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_restrict", BddRestrictCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_compose", BddComposeCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_out_BDD", BddOutBDDCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_out_table", BddOutTableCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_out_function", BddOutFunctionCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_draw_BDD", BddDrawBDDCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cmp", BddCmpCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_find", BddFindCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_support", BddSupportCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_table_size", BddTableSizeCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_table_number", BddTableNumberCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_table_fresh", BddTableFreshCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_table_bad", BddTableBadCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_table_max", BddTableMaxCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_table_generated", BddTableGeneratedCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_block_number", BddBlockNumberCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_garbage_number", BddGarbageNumberCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_swap_number", BddSwapNumberCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_sifting_number", BddSiftingNumberCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_list_used", BddListUsedCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_list_max_length", BddListMaxLengthCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_list_avg_length", BddListAvgLengthCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_ITE_search", BddIteCacheSearchCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_ITE_find", BddIteCacheFindCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_ITE_overwrite", BddIteCacheOverwriteCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_RelOp_num", BddRelopCacheNumCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_RelOp_search", BddRelopCacheSearchCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_RelOp_find", BddRelopCacheFindCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_RelOp_overwrite", BddRelopCacheOverwriteCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_RelOpSimple_num", BddRelopSimpleCacheNumCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_RelOpSimple_search", BddRelopSimpleCacheSearchCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_RelOpSimple_find", BddRelopSimpleCacheFindCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_RelOpSimple_overwrite", BddRelopSimpleCacheOverwriteCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_RelOpComplex_num", BddRelopComplexCacheNumCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_RelOpComplex_search", BddRelopComplexCacheSearchCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_RelOpComplex_find", BddRelopComplexCacheFindCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_RelOpComplex_overwrite", BddRelopComplexCacheOverwriteCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_StabRS_search", BddStabRSCacheSearchCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_StabRS_find", BddStabRSCacheFindCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_cache_StabRS_overwrite", BddStabRSCacheOverwriteCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_node_number", BddNodeNumberCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_node_number_plain", BddNodeNumberPlainCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_node_max_level", BddNodeMaxLevelCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_node_avg_level", BddNodeAvgLevelCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_variable_number", BddVariableNumberCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  Tcl_CreateCommand(interp, "bdd_minterm_count", BddMintermCountCmd,
                     (ClientData) NULL, (Tcl_CmdDeleteProc *) NULL);

  if (Tcl_PkgProvideEx(interp, "est-bdd", "1.0", &bddStubs) == TCL_OK &&
      Tcl_PkgProvideEx(interp, "estsh-bdd", "1.0", &bddStubs) == TCL_OK)
  return TCL_OK; else return TCL_ERROR;
}

#ifdef __cplusplus
}
#endif

/*-----------------------------------------------------------------------*/
/* Definition of internal functions                                      */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Definition of static functions                                        */
/*-----------------------------------------------------------------------*/

/**Function****************************************************************
  Synopsis    [Function BddInitPkgCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddInitPkgCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                   USECONST char **argv)
{
  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

#ifdef USESOCKETS
  if (gui_outputStream == GUIERROR) {
    fprintf(stderr,"(BDD) ERROR: gui socket not initialized\n");
    return TCL_ERROR;
  }
#endif

  printf("Initialization of BDD package... ");
  Bdd_InitPkg();
  printf("OK\n");

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddExitPkgCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddExitPkgCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                   USECONST char **argv)
{
  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  printf("Exit BDD package... ");
  Bdd_ExitPkg();
  printf("OK\n");

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddAboutPkgCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddAboutPkgCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                    USECONST char **argv)
{
  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  Bdd_AboutPkg();

  /* DEBUG */
  /*
  printf("DEBUG: FALSE = %x, TRUE = %x\n",FALSE,TRUE);
  printf("DEBUG: bdd_termTrue = (%p, %x)\n",bdd_termTrue.p,bdd_termTrue.mark);
  printf("DEBUG: bdd_termFalse = (%p, %x)\n",bdd_termFalse.p,bdd_termFalse.mark);
  printf("DEBUG: bdd_termNull = (%p, %x)\n",bdd_termNull.p,bdd_termNull.mark);
  printf(NEWLINE);
  */

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddTerminalsCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddTerminalsCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                     USECONST char **argv)
{
  Est_String s1;
  Est_String name;
  int n,k;

  if (argc != 2) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  n = wordNumber(s1);

  name = strtok(s1," ");  /* first terminal name */
  Bdd_FoaTerminal(name);

  for(k=1;k<n; k++){
    name = strtok(NULL," ");  /* next terminals name */
    Bdd_FoaTerminal(name);
  }

  free(s1);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddEval1Cmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddEval1Cmd(ClientData clientData, Tcl_Interp *interp, int argc,
               USECONST char **argv)
{
  Est_String s1,s2;
  Bdd_Edge f;

  if (argc != 3) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  s2 = strdup(argv[2]);

  f = Bdd_Eval1(s1);
  Bdd_SaveFormula(NULL,s2,f); /* save result */

  free(s1);
  free(s2);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddEval2Cmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddEval2Cmd(ClientData clientData, Tcl_Interp *interp, int argc,
               USECONST char **argv)
{
  Est_String s1,s2;
  Bdd_Edge f;

  if (argc != 3) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  s2 = strdup(argv[2]);

  f = Bdd_Eval2(s1);
  Bdd_SaveFormula(NULL,s2,f); /* save result */

  free(s1);
  free(s2);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddNOTCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddNOTCmd(ClientData clientData, Tcl_Interp *interp, int argc,
               USECONST char **argv)
{
  Est_String s1,s2;
  Bdd_Edge f,r;

  if (argc != 3){
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  s2 = strdup(argv[2]);

  if (Bdd_FindFormula(s1,&f)) {

    r = Bdd_NOT(f);
    Bdd_SaveFormula(NULL,s2,r); /* save result */

  } else printf("Missing Boolean function %s.\n",s1);

  free(s1);
  free(s2);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddITECmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddITECmd(ClientData clientData, Tcl_Interp *interp, int argc,
               USECONST char **argv)
{
  Est_String s1,s2,s3,s4;
  Bdd_Edge f,g,h,r;

  if (argc != 5){
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  s2 = strdup(argv[2]);
  s3 = strdup(argv[3]);
  s4 = strdup(argv[4]);

  if ( Bdd_FindFormula(s1,&f) && Bdd_FindFormula(s2,&g) &&
       Bdd_FindFormula(s3,&h) ) {

    r = Bdd_ITE(f,g,h);
    Bdd_SaveFormula(NULL,s4,r); /* save result */

  } else printf("Missing Boolean function.\n");

  free(s1);
  free(s2);
  free(s3);
  free(s4);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddECmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddECmd(ClientData clientData, Tcl_Interp *interp, int argc,
             USECONST char **argv)
{
  Est_String s1,s2,s3;
  Bdd_Edge f,sup;
  Est_String name;
  int n,k;

  if (argc != 4){
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  s2 = strdup(argv[2]);
  s3 = strdup(argv[3]);

  if (Bdd_FindFormula(s2,&f)) {

    n = wordNumber(s1);

    name = strtok(s1," ");  /* first terminal name */
    sup = Bdd_FoaTerminal(name);
    f = Bdd_E(f,Bdd_GetVariable(sup));

    for(k=1;k<n; k++){
      name = strtok(NULL," ");  /* next terminals name */
      sup = Bdd_FoaTerminal(name);
      f = Bdd_E(f,Bdd_GetVariable(sup));
    }

    Bdd_SaveFormula(NULL,s3,f); /* save result */

  } else printf("Non-existing Boolean function.\n");

  free(s1);
  free(s2);
  free(s3);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddACmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddACmd(ClientData clientData, Tcl_Interp *interp, int argc,
             USECONST char **argv)
{
  Est_String s1,s2,s3;
  Bdd_Edge f,sup;
  Est_String name;
  int n,k;

  if (argc != 4){
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  s2 = strdup(argv[2]);
  s3 = strdup(argv[3]);

  if (Bdd_FindFormula(s2,&f)) {

    n = wordNumber(s1);

    name = strtok(s1," ");  /* first terminal name */
    sup = Bdd_FoaTerminal(name);
    f = Bdd_A(f,Bdd_GetVariable(sup));

    for(k=1;k<n; k++){
      name = strtok(NULL," ");  /* next terminals name */
      sup = Bdd_FoaTerminal(name);
      f = Bdd_A(f,Bdd_GetVariable(sup));
    }

    Bdd_SaveFormula(NULL,s3,f); /* save result */

  } else printf("Non-existing Boolean function.\n");

  free(s1);
  free(s2);
  free(s3);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddRestrictCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddRestrictCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                    USECONST char **argv)
{
  Est_String s1,s2,s3,s4;
  Bdd_Edge f,sup;
  int b;

  if (argc != 5){
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  s2 = strdup(argv[2]);
  s3 = strdup(argv[3]);
  s4 = strdup(argv[4]);

  sscanf(s3,"%d",&b);

  if (Bdd_FindFormula(s1,&f)) {

    sup = Bdd_FoaTerminal(s2);
    if (b == 0) f = Bdd_Restrict(f,Bdd_GetVariable(sup),FALSE);
      else f = Bdd_Restrict(f,Bdd_GetVariable(sup),TRUE);
    Bdd_SaveFormula(NULL,s4,f); /* save result */

  } else printf("Missing Boolean function.\n");

  free(s1);
  free(s2);
  free(s3);
  free(s4);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddComposeCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddComposeCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                   USECONST char **argv)
{
  Est_String s1,s2,s3,s4;
  Bdd_Edge f,g,r,sup;

  if (argc != 5){
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  s2 = strdup(argv[2]);
  s3 = strdup(argv[3]);
  s4 = strdup(argv[4]);

  if ( Bdd_FindFormula(s1,&f) && Bdd_FindFormula(s3,&g) ) {

    sup = Bdd_FoaTerminal(s2);
    r = Bdd_Compose(f,Bdd_GetVariable(sup),g);
    Bdd_SaveFormula(NULL,s4,r); /* save result */

  } else printf("Missing Boolean function.\n");

  free(s1);
  free(s2);
  free(s3);
  free(s4);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddOutBDDCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddOutBDDCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                  USECONST char **argv)
{
  Est_String s1;

  if (argc != 2) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  printf("BDD %s written to stdout.\n",s1);
  Bdd_WriteBDD(s1);

  free(s1);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddOutTableCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddOutTableCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                  USECONST char **argv)
{
  Est_String s1;

  if (argc != 2) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  printf("Table for BDD %s written to stdout.\n",s1);
  Bdd_WriteTable(s1);

  free(s1);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddOutFunctionCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddOutFunctionCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                       USECONST char **argv)
{
  Est_String s1;

  if (argc != 2) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  Bdd_WriteFunction(stdout,s1);

  free(s1);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddDrawBDDCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddDrawBDDCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                  USECONST char **argv)
{
  Est_String funname;
  FILE *funfile;
  Est_String s1;
  int ERR;
#ifndef ESTWIN32
  pid_t childpid;
#endif
#ifdef ESTWIN32
  char* docdir;
  char* appdir;
  docdir = getenv("USERPROFILE");
  appdir = strdup("\\AppData\\Local\\");
  char* bddviewtcl;
#endif

  /* TESTING */
  /*
  Est_String xmlname;
  FILE *xmlfile;
  */

  if (argc != 2) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

#ifdef ESTWIN32
  funname = (Est_String) malloc(strlen(docdir)+strlen(appdir)+11+1);
  funname[0] = 0;
  strcat(funname,docdir);
  strcat(funname,appdir);
  strcat(funname,"bdd.bddview");
  free(appdir);
#elif ESTUNIX
  funname = strdup("/tmp/XXXXXX");
  close(mkstemp(funname));
#else
  funname = strdup("bdd.bddview");
#endif

  funfile = fopen(funname,"w");
  if (!funfile) {
    printf("File error (%s)\n",funname);
    Tcl_SetResult(interp, "", TCL_STATIC);
    return TCL_ERROR;
  }

  /* TESTING */
  /*
  xmlname = strdup("bdd.graphml");
  xmlfile = fopen(xmlname,"w");
  */

  ERR = Bdd_WriteBDDView(funfile,s1,-1);
  fclose(funfile);

  /* TESTING */
  /*
  Bdd_WriteGraphML(xmlfile,s1);
  fclose(xmlfile);
  */

  free(s1);

  if (!ERR) {

#ifdef ESTWIN32
    bddviewtcl = NULL;
    if (_access("bddview.tcl",0) == 0 ) {
      bddviewtcl = strdup("bddview.tcl");
    }
    else if (_access("C:/msys64/home/Robert/biddy/bddview/bddview.tcl",0) == 0 ) {
      bddviewtcl = strdup("C:/msys64/home/Robert/biddy/bddview/bddview.tcl");
    }

    if (bddviewtcl) {
      if (_spawnlp(_P_WAIT,"wish.exe","wish.exe",bddviewtcl,funname,NULL) == -1) {
        printf("Error (eerno=%d): wish.exe is not executable (check your path!)\n",errno);
      }
      free(bddviewtcl);
    } else {
      printf("Error: missing bddview.tcl, try changing current directory to est2ed/bin\n");
    }
#else
    childpid = fork();
    if (childpid == 0) {
      execlp("./bddview.tcl","bddview.tcl",funname,NULL);
      execlp("/usr/lib/est1ed/bddview.tcl","bddview.tcl",funname,NULL);
      execlp("/usr/lib/est2ed/bddview.tcl","bddview.tcl",funname,NULL);
      execlp("../bin/bddview.tcl","bddview.tcl",funname,NULL);
      execlp("../../bin/bddview.tcl","bddview.tcl",funname,NULL);
      printf("bddview.tcl not found\n");
    }
#endif

  }
  
  free(funname);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddCmpCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddCmpCmd(ClientData clientData, Tcl_Interp *interp, int argc,
               USECONST char **argv)
{
  Est_String s1,s2;
  Bdd_Edge f,g;
  Est_String sup = NULL;

  if (argc != 3){
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  s2 = strdup(argv[2]);

  if ( Bdd_FindFormula(s1,&f) && Bdd_FindFormula(s2,&g) ) {

    if (Bdd_isEqv(f,g)) {
      sup = (Est_String) strdup("TRUE");
    } else {
      sup = (Est_String) strdup("FALSE");
    }

  } else {
    sup = (Est_String) strdup("Missing Boolean function.");
  }

  free(s1);
  free(s2);

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  if (sup) free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddFindCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddFindCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                USECONST char **argv)
{
  Est_String s1;
  Bdd_Edge f;
  Est_String sup;

  if (argc != 2){
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  if ( Bdd_FindFormula(s1,&f)) {
    sup = (Est_String) strdup("TRUE");
  } else {
    sup = (Est_String) strdup("FALSE");
  }

  free(s1);

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddSupportCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddSupportCmd(ClientData clientData, Tcl_Interp *interp, int argc,
               USECONST char **argv)
{
  Est_String s1,s2;
  Bdd_Edge f,r;

  if (argc != 3){
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);
  s2 = strdup(argv[2]);

  if (Bdd_FindFormula(s1,&f)) {

    r = Bdd_Support(f);
    Bdd_SaveFormula(NULL,s2,r); /* save result */

  } else printf("Missing Boolean function %s.\n",s1);

  free(s1);
  free(s2);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  Tcl_SetResult(interp, "", TCL_STATIC);
  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddTableSizeCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddTableSizeCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                     USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%u",BddNodeTableSize());
  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}


/**Function****************************************************************
  Synopsis    [Function BddTableNumberCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddTableNumberCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                       USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%u",BddNodeTableNum());
  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddTableFreshCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddTableFreshCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(32);
  sprintf(sup,"%s","NOT IMPLEMENTED");
  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddTableBadCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddTableBadCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                    USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(32);
  sprintf(sup,"%s","NOT IMPLEMENTED");
  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddTableMaxCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddTableMaxCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                    USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%u",BddNodeTableMax());
  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddTableGeneratedCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddTableGeneratedCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                          USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%u",BddNodeTableGenerated());
  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddBlockNumberCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddBlockNumberCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                         USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%u",BddNodeTableBlockNumber());
  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddGarbageNumberCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddGarbageNumberCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                         USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%u",BddNodeTableGarbage());
  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddSwapNumberCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddSwapNumberCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                         USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%u",BddNodeTableSwapNumber());
  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddSiftingNumberCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddSiftingNumberCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                         USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%u",BddNodeTableSiftingNumber());
  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddListUsedCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddListUsedCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                    USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%u",BddListUsed());

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddListMaxLengthCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddListMaxLengthCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                         USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%u",BddListMaxLength());

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddListAvgLengthCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddListAvgLengthCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                         USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);
  sprintf(sup,"%.2f",BddListAvgLength());

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddIteCacheSearchCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddIteCacheSearch()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddIteCacheSearch());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddIteCacheFindCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddIteCacheFind()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddIteCacheFind());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddIteCacheOverwriteCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddIteCacheOverwrite()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddIteCacheOverwrite());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddRelopCacheNumCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddRelopCacheNum()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddRelopCacheNum());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddRelopCacheSearchCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddRelopCacheSearch()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddRelopCacheSearch());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddRelopCacheFindCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddRelopCacheFind()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddRelopCacheFind());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddRelopCacheOverwriteCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddRelopCacheOverwrite()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddRelopCacheOverwrite());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddRelopSimpleCacheNumCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddRelopSimpleCacheNum()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddRelopSimpleCacheNum());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddRelopSimpleCacheSearchCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddRelopSimpleCacheSearch()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddRelopSimpleCacheSearch());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddRelopSimpleCacheFindCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddRelopSimpleCacheFind()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddRelopSimpleCacheFind());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddRelopSimpleCacheOverwriteCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddRelopSimpleCacheOverwrite()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddRelopSimpleCacheOverwrite());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddRelopComplexCacheNumCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddRelopComplexCacheNum()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddRelopComplexCacheNum());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddRelopComplexCacheSearchCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddRelopComplexCacheSearch()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddRelopComplexCacheSearch());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddRelopComplexCacheFindCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddRelopComplexCacheFind()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddRelopComplexCacheFind());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddRelopComplexCacheOverwriteCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddRelopComplexCacheOverwrite()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddRelopComplexCacheOverwrite());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddStabRSCacheSearchCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddStabRSCacheSearch()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddStabRSCacheSearch());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddStabRSCacheFindCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddStabRSCacheFind()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddStabRSCacheFind());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddStabRSCacheOverwriteCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String sup;

  if (argc != 1) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  sup = (Est_String) malloc(15);

#ifdef ESTWIN32
  sprintf(sup,"%I64u",BddStabRSCacheOverwrite()); /* MINGW REQUIRES "%I64u" */
#else
  sprintf(sup,"%llu",BddStabRSCacheOverwrite());
#endif

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddNodeNumberCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddNodeNumberCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String s1;
  unsigned int i;
  Bdd_Edge f;
  Est_String sup;

  if (argc != 2) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  if (Bdd_FindFormula(s1,&f)) {

    i = BddNodeNumber(f);
    sup = (Est_String) malloc(15);
    sprintf(sup,"%u",i);

  } else {
    printf("Missing Boolean function.\n");
    sup = strdup("");
  }

  free(s1);

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddNodeNumberPlainCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddNodeNumberPlainCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String s1;
  unsigned int i;
  Bdd_Edge f;
  Est_String sup;

  if (argc != 2) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  if (Bdd_FindFormula(s1,&f)) {

    i = BddNodeNumberPlain(f);
    sup = (Est_String) malloc(15);
    sprintf(sup,"%u",i);

  } else {
    printf("Missing Boolean function.\n");
    sup = strdup("");
  }

  free(s1);

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddNodeMaxLevelCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddNodeMaxLevelCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                        USECONST char **argv)
{
  Est_String s1;
  unsigned int i;
  Bdd_Edge f;
  Est_String sup;

  if (argc != 2) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  if (Bdd_FindFormula(s1,&f)) {

    i = BddNodeMaxLevel(f);
    sup = (Est_String) malloc(15);
    sprintf(sup,"%u",i);

  } else {
    printf("Missing Boolean function.\n");
    sup = strdup("");
  }

  free(s1);

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddNodeAvgLevelCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddNodeAvgLevelCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                        USECONST char **argv)
{
  Est_String s1;
  float d;
  Bdd_Edge f;
  Est_String sup;

  if (argc != 2) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  if (Bdd_FindFormula(s1,&f)) {

    d = BddNodeAvgLevel(f);
    sup = (Est_String) malloc(15);
    sprintf(sup,"%.2f",d);

  } else {
    printf("Missing Boolean function.\n");
    sup = strdup("");
  }

  free(s1);

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddVariableNumberCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddVariableNumberCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                      USECONST char **argv)
{
  Est_String s1;
  unsigned int i;
  Bdd_Edge f;
  Est_String sup;

  if (argc != 2) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  if (Bdd_FindFormula(s1,&f)) {

    i = BddVariableNumber(f);
    sup = (Est_String) malloc(15);
    sprintf(sup,"%u",i);

  } else {
    printf("Missing Boolean function.\n");
    sup = strdup("");
  }

  free(s1);

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    [Function BddMintermCountCmd.]
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
BddMintermCountCmd(ClientData clientData, Tcl_Interp *interp, int argc,
                        USECONST char **argv)
{
  Est_String s1,s2;
  Bdd_Edge f;
  int nvars;
  double d;
  Est_String sup;

  if ((argc != 2) && (argc != 3)) {
    Tcl_SetResult(interp, "wrong # args", TCL_STATIC);
    return TCL_ERROR;
  }

  s1 = strdup(argv[1]);

  if (argc == 3) {
    s2 = strdup(argv[2]);
    sscanf(s2,"%d",&nvars);
    free(s2);
  } else {
    nvars = 0;
  }

  if (Bdd_FindFormula(s1,&f)) {

    d = BddMintermCount(f,nvars);
    sup = (Est_String) malloc(15);
    sprintf(sup,"%.0f",d);

  } else {
    printf("Missing Boolean function.\n");
    sup = strdup("");
  }

  free(s1);

  Tcl_SetResult(interp, sup, TCL_VOLATILE);
  free(sup);

#ifdef USESOCKETS
  if (gui_outputLength == -2) /* remote server */
  printf("EST_TCL_OK\n");
#endif

  return TCL_OK;
}

/**Function****************************************************************
  Synopsis    []
  Description []
  SideEffects []
  SeeAlso     []
  ************************************************************************/

static int
wordNumber(Est_String s)
{
  int num;
  int token;
  Est_String sup,sup_next;

  num = 0;
  sup = strdup(s);

  token = strspn(sup," ");
  while (strspn(sup," ") < strlen(sup)) {
    sup_next = (Est_String) strdup(&sup[token]);
    free(sup);
    sup = sup_next;

    num++;
    token = strcspn(sup," ");
    sup_next = (Est_String) strdup(&sup[token]);
    free(sup);
    sup = sup_next;

    token = strspn(sup," ");
  }

  free(sup);
  return num;
}
