#  Authors     [Robert Meolic (robert.meolic@um.si)]
#  Revision    [$Revision: 127 $]
#  Date        [$Date: 2016-07-14 02:22:29 +0200 (čet, 14 jul 2016) $]
#
#  Copyright   [This file is part of EST (Efficient Symbolic Tools).
#               Copyright (C) 2003, 2016
#               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia
#
#               EST is free software; you can redistribute it and/or modify
#               it under the terms of the GNU General Public License as
#               published by the Free Software Foundation; either version 2
#               of the License, or (at your option) any later version.
#
#               EST is distributed in the hope that it will be useful,
#               but WITHOUT ANY WARRANTY; without even the implied warranty of
#               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
#               GNU General Public License for more details.
#
#               You should have received a copy of the GNU General Public
#               License along with this program; if not, write to the Free
#               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
#               Boston, MA 02110-1301 USA.]

# Here, version 1.0 should be given due to the problems with
# "package ifneeded" used in pkgIndex.tcl in the packages that use this.
package provide est-server 1.0

# ####################################################################
# Socket server is used to catch output strings
# GUIOUTPUTLENGTH - how many bytes is waiting in the channel
# GUIOUTPUTLENGTHOK - how many bytes is already written out
# if (GUIOUTPUTLENGTH < 0) then counting bytes is disabled
# Shared variable GUIOUTPUTLENGTH is also used to denote remote server!
# ####################################################################

proc est_accept {startTime s addr port} {
  puts "Created channel $s to accept from socket addr $addr port $port."
  fconfigure $s -blocking 0
  fileevent $s readable [list est_read $s $startTime]
}

proc est_read {s startTime} {
  global GUIOUTPUTLENGTH
  global GUIOUTPUTLENGTHOK
  if {[eof $s] || [catch {gets $s line}]} {
    close $s
    puts "Channel closed."
  } else {
    if {[fblocked $s]} {
      #puts "BLOCKED([chan pending input $s])"
    } else {
      if {$GUIOUTPUTLENGTH < 0} {
        # counting bytes mechanism not used
        if {$line == ""} {
          outline_direct ""
        } else {
          out_direct $line
        }
      } else {
        # counting bytes mechanism used
        if {($line == "") && ($GUIOUTPUTLENGTH > 0)} {
            outline_direct ""
            incr GUIOUTPUTLENGTHOK 1
            incr GUIOUTPUTLENGTH [expr -1 * $GUIOUTPUTLENGTHOK]
            set GUIOUTPUTLENGTHOK 0
        } else {
          out_direct $line
          incr GUIOUTPUTLENGTHOK [string length $line]
        }
        #puts "$line ($GUIOUTPUTLENGTH,$GUIOUTPUTLENGTHOK)"
      }
    }
  }
}
