/**CHeaderFile*************************************************************
  PackageName [gui]
  Synopsis    [Package 'gui' is used to redirect output to GUI.]

  FileName    [gui.h]
  Revision    [$Revision: 113 $]
  Date        [$Date: 2016-03-08 22:31:16 +0100 (tor, 08 mar 2016) $]
  Authors     [Robert Meolic (robert.meolic@um.si)]
  Description [File gui.h contains declaration of all external
               data structures.]
  SeeAlso     [guiInt.h]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 2003, 2016
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

#ifndef _GUI
#define _GUI

#include <est.h>

/* ON WIN32 THERE HAS TO BE DEFINED ESTWIN32 */
/* ON UNIX THERE HAS TO BE DEFINED ESTUNIX */
/* ON MACOSX THERE HAS TO BE DEFINED ESTMACOSX */

#ifdef ESTUNIX
#  ifndef EXTERN
#    define EXTERN extern
#  endif
#  ifndef DATAEXTERN
#    define DATAEXTERN extern
#  endif
#endif

#ifdef ESTMACOSX
#  ifndef EXTERN
#    define EXTERN extern
#  endif
#  ifndef DATAEXTERN
#    define DATAEXTERN extern
#  endif
#endif

#ifdef ESTWIN32
#  ifdef TCL
#    undef EXTERN
#    define EXTERN extern
#    undef DATAEXTERN
#    define DATAEXTERN extern
#  else
#    if defined(BUILD_EST) || defined(BUILD_GUI)
#      undef EXTERN
#      define EXTERN __declspec (dllexport)
#      undef DATAEXTERN
#      define DATAEXTERN extern __declspec (dllexport)
#    else
#      if defined(USE_EST) || defined(USE_GUI)
#        undef EXTERN
#        define EXTERN __declspec (dllimport)
#        undef DATAEXTERN
#        define DATAEXTERN __declspec (dllimport)
#      else
#        undef EXTERN
#        define EXTERN extern
#        undef DATAEXTERN
#        define DATAEXTERN extern
#      endif
#    endif
#  endif
#endif

/*-----------------------------------------------------------------------*/
/* Constant definitions                                                  */
/*-----------------------------------------------------------------------*/

/*-----------------------------------------------------------------------*/
/* Macro definitions                                                     */
/*-----------------------------------------------------------------------*/

#ifdef USESOCKETS
#  ifdef ESTWIN32
#    include <io.h>
#    include <fcntl.h>
#    include <winsock2.h>
#    include <ws2tcpip.h>
#    define Gui_Output SOCKET
#    define GUIERROR INVALID_SOCKET
#  else
#    include <sys/socket.h>
#    include <arpa/inet.h>
#    include <netdb.h>
#    define Gui_Output FILE *
#    define GUIERROR NULL
#  endif
#else
#  define Gui_Output int
#  define GUIERROR 0
#endif

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

DATAEXTERN int gui_status;
DATAEXTERN Gui_Output gui_outputStream;
DATAEXTERN int gui_outputLength;

#ifdef __cplusplus
}
#endif

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Function prototypes                                                   */
/*-----------------------------------------------------------------------*/

#ifdef __cplusplus
extern "C" {
#endif

EXTERN void Gui_InitPkg();

EXTERN void Gui_ExitPkg();

EXTERN void Gui_AboutPkg();

EXTERN void Gui_Update();

#ifdef __cplusplus
}
#endif

/**AutomaticEnd***********************************************************/

#endif  /* _GUI */
