/**CHeaderFile*************************************************************
  PackageName [versis]
  Synopsis    [Package 'versis' provides basic tools for formal
               verification of concurrent systems]

  FileName    [versisInt.h]
  Revision    [$Revision: 113 $]
  Date        [$Date: 2016-03-08 22:31:16 +0100 (tor, 08 mar 2016) $]
  Authors     [Robert Meolic (robert.meolic@um.si),
               Mirjam Sepesy Maucec (mirjam.sepesy@um.si),
               Tatjana Kapus (tatjana.kapus@um.si)]
  Description [File versisInt.h contains declaration of internal
               data structures.]
  SeeAlso     [versis.h]

  Copyright   [This file is part of EST (Efficient Symbolic Tools).
               Copyright (C) 2003, 2016
               UM-FERI, Smetanova ulica 17, SI-2000 Maribor, Slovenia

               EST is free software; you can redistribute it and/or modify
               it under the terms of the GNU General Public License as
               published by the Free Software Foundation; either version 2
               of the License, or (at your option) any later version.

               EST is distributed in the hope that it will be useful,
               but WITHOUT ANY WARRANTY; without even the implied warranty of
               MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
               GNU General Public License for more details.

               You should have received a copy of the GNU General Public
               License along with this program; if not, write to the Free
               Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
               Boston, MA 02110-1301 USA.]
  ************************************************************************/

#ifndef _VERSISINT
#define _VERSISINT

#include "versis.h"

#ifdef USE_BDD_STUBS
#  include <bddDecls.h>
#endif

#ifdef USE_PA_STUBS
#  include <paDecls.h>
#endif

/* if using sockets then printf("\n") will produce two new lines */
/* you will avoid this efect by using printf(NEWLINE) which adds extra space */
/* this extra space is not needed if counting bytes mechanism is used */
#ifdef USESOCKETS
#  include <gui.h>
#  ifdef USE_GUI_STUBS
#    include <guiDecls.h>
#  endif
#  ifdef ESTWIN32
#    define printf(...) {char sendbuf[1024]; sprintf(sendbuf,__VA_ARGS__);if(gui_outputLength<0){send(gui_outputStream,sendbuf,(int)strlen(sendbuf),0);}else{gui_outputLength+=send(gui_outputStream,sendbuf,(int)strlen(sendbuf),0);}send(gui_outputStream,"\n",1,0);Gui_Update();}
#  else
#    define printf(...) {if(gui_outputLength<0){fprintf(gui_outputStream,__VA_ARGS__);}else{gui_outputLength+=fprintf(gui_outputStream,__VA_ARGS__);}fprintf(gui_outputStream,"\n");Gui_Update();}
#    define debugprintf(...) {fprintf(stderr,"VERSIS[");fprintf(stderr,__VA_ARGS__);fprintf(stderr,"]");if(gui_outputLength<0){fprintf(gui_outputStream,__VA_ARGS__);}else{gui_outputLength+=fprintf(gui_outputStream,__VA_ARGS__);}fprintf(gui_outputStream,"\n");Gui_Update();}
#  endif
#  define NEWLINE " \n"
#else
#  define NEWLINE "\n"
#endif

/*-----------------------------------------------------------------------*/
/* Type declarations                                                     */
/*-----------------------------------------------------------------------*/

typedef int *TTabIndexov;   /* tabela indeksov spremenljivk */

/*-----------------------------------------------------------------------*/
/* Structure declarations                                                */
/*-----------------------------------------------------------------------*/

typedef struct {
  Bdd_Edge S;
  Est_Boolean b;
  int t;
} VersisST;

typedef struct {
  int num;
  VersisST *table;
} VersisStateTable;

/*-----------------------------------------------------------------------*/
/* Variable declarations                                                 */
/*-----------------------------------------------------------------------*/

/**AutomaticStart*********************************************************/

/*-----------------------------------------------------------------------*/
/* Function prototypes                                                   */
/*-----------------------------------------------------------------------*/

extern void Versis_Compose_Thr(Est_String name, int type, int numP,
   Est_String process, int numA, Est_String actions, Est_Boolean onthefly);

extern Est_Boolean Versis_Equivalence_Thr(int type, int par1,
                           Est_String name1, int par2, Est_String name2);

extern void Versis_Minimization_Thr(int type, int par, Est_String name);

extern Est_Boolean VersisStrongEquivalence(Bdd_Edge *eqv,
                                      int par1, int n1, int par2, int n2);

extern Est_Boolean VersisWeakEquivalence(Bdd_Edge *eqv,
                                      int par1, int n1, int par2, int n2);

extern Est_Boolean VersisTestEquivalence(Bdd_Edge *eqv,
                                      int par1, int n1, int par2, int n2,
                                      int *acc1, int *acc2);

extern Est_Boolean VersisTraceEquivalence(int type, Bdd_Edge *eqv,
                                      int par1, int n1, int par2, int n2,
                                      int *det1, int *det2);

extern int VersisStrongMinimization(Est_String name, int par, int n);

extern int VersisWeakMinimization(Est_String name, int par, int n);

extern int VersisTraceMinimization(int type, Est_String name, int par, int n);

extern int VersisWCAMinimization(Est_String name, int par, int n);

extern Bdd_Edge VersisBuildDet(Est_String orgname, Est_String newname,
			Bdd_Edge init, Bdd_Edge Ptau, Bdd_Edge trcl, int s);

extern Bdd_Edge VersisTrCl(Bdd_Edge f);

extern Bdd_Edge VersisComposeActions(Pa_Composition *c);

/**AutomaticEnd***********************************************************/

#endif /* _VERSISINT */











